var currentUser;

window.addEventListener('click', function(event){
	if (event.target.tagName == "A") {
		if (event.target.href.startsWith("javascript:")) {
			return
		}
		event.target.href = appendWho(event.target.href)
	}
	if(event.target.classList.contains('depth-toggle')){
		toggleDepth(event);
	}
}, true);

window.addEventListener('load', function(event){
	var ws = document.getElementById("who-select")
	if (ws) {
		var urlParams = new URLSearchParams(window.location.search);
		const whoParam = urlParams.get("who");
		if (whoParam) {
			currentUser = {
				Id: parseInt(whoParam, 10),
				GeneratedDisplayname: ws.value
			}
		}
	}

	// Fill the "section" menu & add anchors
	var menu = document.getElementById("menu")
	if (menu) {

		items = []
		for (var item1 of document.querySelectorAll("h2,h3")) {
			items.push(item1)
		}

		menu.onchange = function() {
			location.hash = "#"+menu.value
		}

		var opt = document.createElement("option")
		// empty option
		menu.appendChild(opt)
		var menuI = 0
		for (var item of items) {
			var iter = document.createNodeIterator(item, NodeFilter.SHOW_TEXT)
			var firstText = iter.nextNode()
			if (!firstText) {
				continue
			}
			var text = firstText.textContent.trim()

			// Add anchor to the h2
			var anchor = document.createElement("a")
			anchor.name = text
			anchor.classList.add("anchor")
			item.appendChild(anchor)

			// Add options to the menu
			var opt = document.createElement("option")
			if (item.nodeName == "H3") {
				// add left padding to H3 anchors
				opt.innerText = "   "+text // the "spaces" here are non-breaking!
			} else {
				// no padding for H2 anchors
				opt.innerText = text
			}
			opt.value = text

			if (decodeURIComponent(location.hash) == "#"+text) {
				opt.selected = true
				location.hash = "#"
				location.hash = "#"+encodeURIComponent(text)
			}

			menu.appendChild(opt)
		}

		if (menu.childNodes.length > 1) {
			menu.parentNode.style.display = ""
		}
	}
});

/**
* Unai 09.06.2020: Changed this as it was hiding/showing all the fields below depth
* Therefore everything was expanded/collapsed and the handle arrows showing incorrectly
* @function toggleDepth toggles the arrow handles and makes rows below visible/invisible
* @param {Event} event
*/
function toggleDepth(event){

	// Prevent default/propagation
	event.preventDefault();
	event.stopPropagation();

	// Check whether it was clicked with shift, it will show/hide all children
	var toggleAll = event.shiftKey;

	// Get handle/arrow parent node, which holds the state
	var rowEl = event.target.parentNode && event.target.parentNode.parentNode;

	// Get current depth as numeric value for proper comparison
	var depth = rowEl && rowEl.dataset.depth && parseInt(rowEl.dataset.depth, 10) || 0;

	// Set to hide or show depending on current state/class
	var hide = !event.target.classList.contains('is-closed');

	// Trigger recursive toggle
	toggleDepthLevel(rowEl, hide, toggleAll, depth)
}

/**
* @function toggleDepthLevel toggles single level arrow handle and makes rows below visible/invisible
* @param {HTMLElement} rowEl the row element
* @param hide whether the action is to show or hide
* @param toggleAll whether to toggle all modifer is applied or not
* @param initialDepth the depth everything was originated from, to perform special action on node
*/
function toggleDepthLevel(rowEl, hide, toggleAll, initialDepth){

	// Get reference to to handle/arrow, if any
	var handleEl = rowEl && rowEl.querySelector('span.depth-toggle');

	// If there's no handle/arrow, skip
	if (!handleEl) {
		return
	}

	// Get current depth as numeric value for proper comparison
	var depth = rowEl && rowEl.dataset.depth && parseInt(rowEl.dataset.depth, 10) || 0;

	// Check whether this evel rows were shown or hidden
	var hidden = handleEl.classList.contains('is-closed');

	// Toggle current handle arrow to 'hide' value if:
	// - it is the initial row
	// - toggleAll modifier is on
	if (depth === initialDepth || toggleAll) {
		handleEl.classList.toggle('is-closed', hide);
	}

	// Get initial next row element and its depth
	var nextRowEl = rowEl.nextElementSibling;
	var nextDepth = nextRowEl && nextRowEl.dataset.depth && parseInt(nextRowEl.dataset.depth, 10) || 0;

	// Loop over rows elements, as long as we have sibling elements
	while (nextRowEl && nextDepth) {

		// Only process if it is the immediate depth level, others will get here recursively
		if (nextDepth === depth + 1) {

			// Toggle row visibility if:
			// - it is to hide
			// - toggleAll modifier is on
			// - it is the initial depth
			// - it is to show and it was not hidden before
			if (hide || toggleAll || depth === initialDepth || !hide && !hidden) {
				nextRowEl.classList.toggle('is-hidden', hide);

				// Also, trigger recursive toggle only under these conditions
				toggleDepthLevel(nextRowEl, hide, toggleAll, initialDepth)
			}
		} else if (nextDepth == depth) {
			// We reached a sibling element at the same depth, we stop here
			break;
		}

		// Get next row and depth
		nextRowEl = nextRowEl.nextElementSibling;
		nextDepth = nextRowEl && nextRowEl.dataset.depth && parseInt(nextRowEl.dataset.depth, 10) || 0;
	}
}

function updateQueryStringParameter(uri, key, value) {
	// the "uri" is sometimes only "?who=some", so we cannot
	// use "new URL" here
	const parts = uri.split('?');
	const searchParams = new URLSearchParams(parts[1] || '');
	if (value === undefined) {
		searchParams.delete(key);
	} else {
		searchParams.set(key, value);
	}
	let query = searchParams.toString()
	if (query.length) {
		return parts[0]+"?"+query;
	} else {
		return parts[0]
	}
}

function appendWho(uri) {
	var ws = document.getElementById("who-select")
	if (!ws) {
		return uri
	}
	return updateQueryStringParameter(uri, "who", ws.value || undefined)
}

function goTo(uri, hash = location.hash, evt) {
	if (evt && evt.target != evt.currentTarget && evt.target.parentElement != evt.currentTarget) {
		return;
	}
	// console.warn("GoTo:", appendWho(uri), hash)
	document.body.style.opacity = "0.5"
	if (hash) {
		document.location.href = appendWho(uri)+hash
	} else {
		document.location.href = appendWho(uri)
	}
}

window.addEventListener('load', function(event){
	var timers = document.querySelector('.timers');
	if(timers){
		if(sessionStorage.getItem('timers-open'))
		timers.open = true;

		timers.addEventListener('toggle', function(){
			if(timers.open){
				sessionStorage.setItem('timers-open', 1);
			} else {
				sessionStorage.removeItem('timers-open');
			}
		})
	}

	// Load users on who select click
	let users = [];
	const whoSelectEl = document.querySelector("#who-select");
	if (whoSelectEl) {
		whoSelectEl.addEventListener('click', (evt) => {
			if (!users || users.length < 1) {
				whoSelectEl.innerHTML = `<option value>loading ...</option>`;
				// whoSelectEl.setAttribute("disabled", "true");
				loadUsers((usrs, err) => {
					if (err) {
						return console.error(`Error getting users for who select: ${err.message}`)
					}
					users = usrs.sort((a, b) => a.GeneratedDisplayname.localeCompare(b.GeneratedDisplayname));
					let htmlStr = `<option value></option>`;
					users.forEach((usr) => {
						htmlStr += `<option value="${escape(''+usr.Id)}"`;
						if (currentUser && currentUser.Id === usr.Id) {
							htmlStr += ` selected`;
						}
						htmlStr += `>${escape(usr.GeneratedDisplayname+" ["+usr.Id+"]")}</option>`;
					});
					// whoSelectEl.removeAttribute("disabled");
					whoSelectEl.innerHTML = htmlStr;
					var event = new MouseEvent('click');
					whoSelectEl.dispatchEvent(event);
				});
			}
		});
	}
});

/**
 * @function addUserToWhoSelect adds a user option to the select
 * @param {Object} usr
 */
function addUserToWhoSelect(usr, selected) {
	const whoSelectEl = document.querySelector("#who-select");
	const whoSelectOpts = whoSelectEl.querySelectorAll("option");
	if (Array.prototype.some.call(whoSelectOpts, (elOpt) => `${elOpt.value}` == `${usr.Id}`)) {
		return;
	}
	const opt = document.createElement("option");
	opt.value = usr.Id;
	opt.innerHTML = usr.GeneratedDisplayname;
	if (selected) {
		opt.setAttribute("selected", true);
	}
	whoSelectEl.appendChild(opt);
}

/**
 * @function loadUsers loads users
 * @param {function} cb the callback to be invoked with:
 * - @param {Array<Object>} users array of doXHR
 * - @param {Object} error object with error message, code, etc.
 */
function loadUsers(cb) {

	// Perform specific XHR
	const headers = {
		'Accept': 'application/json'
	};
	const queryString = "limit=9999999&short=1&accept=application/json&pretty=0"
	doXHR("GET", `/inspect/users/?${queryString}`, null, headers, (xhr) => {
		let data;
		try {
			data = JSON.parse(xhr.responseText);
			if (!data) {
				return cb(null, {
					message: `Error fetching users list as JSON: No proper data from response: ${xhr.responseText}`
				});
			}
			cb(data.Users || []);
		} catch(e) {
			cb(null, {
				message: `Error fetching users list as JSON ${e.message}`
			});
		}
	})
}

/**
 * @function loadUser loads users
 * @param {string | number} id the user id
 * @param {function} cb the callback to be invoked with:
 * - @param {Object} user user object
 * - @param {Object} error object with error message, code, etc.
 */
function loadUser(id, cb) {

	// Perform specific XHR
	const headers = {
		'Accept': 'application/json'
	};
	doXHR("GET", `/inspect/users/${id}/`, null, headers, (xhr) => {
		let data;
		try {
			data = JSON.parse(xhr.responseText);
			if (!data) {
				return cb(null, {
					message: `Error fetching user as JSON: No proper data from response: ${xhr.responseText}`
				});
			}
			data.User.GeneratedDisplayname = data.GeneratedDisplayname;
			cb(data.User);
		} catch(e) {
			cb(null, {
				message: `Error fetching user as JSON ${e.message}`
			});
		}
	});
}

/**
 * @function doXHR helper to perform an XMLHTTPRequest
 * @param {string} method the HTTP method
 * @param {string} url the url to call
 * @param {any} data the data to sendm if any
 * @param {Object} headers the key/value for headers
 * @param {function} cb the callback after async operation concludes, it will include
 * - @param {XHRHTTTPRequest} xhr the response, once loaded
 */
function doXHR(method, url, data, headers, cb) {
	const xhr = new XMLHttpRequest();
	xhr.open(method, url);
	xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
	headers = headers || {};
	Object.keys(headers).forEach((h) => {
		xhr.setRequestHeader(h, headers[h]);
	});
	xhr.addEventListener("load", () => cb(xhr));
	xhr.send(data);
}

/**
 * @function escape escapes HTML
 * @param {string} str the string to escape
 */
function escape(str) {
    let lookup = {
        '&': "&amp;",
        '"': "&quot;",
        '<': "&lt;",
        '>': "&gt;"
    };
    return str.replace(/[&"<>]/g, (c) => lookup[c]);
}

/**
 * @function removeQueryStringParameter removes parameters from query string and updates url
 * @param {array} arguments the parameters to remove
 */
function removeQueryStringParameters() {
	if (!arguments.length) {
		return;
	}
	const urlParams = new URLSearchParams(window.location.search);
	for (let i = 0, j = arguments.length; i < j; i++) {
		urlParams.delete(arguments[i]);
	}
	let url = ''
	if (urlParams.toString()) {
		url = `${document.location.origin}${document.location.pathname}?${urlParams.toString()}`;
	} else {
		url = `${document.location.origin}${document.location.pathname}`;
	}
	window.history.replaceState(null, null, url);
}

function setParams() {
	s = window.location.search
	for (let i = 0, j = arguments.length; i < j; i=i+2) {
		key = arguments[i]
		value = arguments[i+1]
		s = setParam(s, key, value)
	}
	return s
}

function setParam(s, key, value) {
	const urlParams = new URLSearchParams(s);
	urlParams.delete(key);
	urlParams.set(key, value)
	return urlParams.toString()
}