document.addEventListener("DOMContentLoaded", (event) => {
    // set brand colors using https://www.vis4.net/chromajs
    // all color manipulation must be copied from webfrontend Session.coffee

    var brand_color = getComputedStyle(document.documentElement).getPropertyValue('--brand-color') || null;

    if (brand_color != null) {
        document.documentElement.classList.add("brand-color-custom");

        // console.log("contrast original brand color: ", brand_color)

        // 1) Hover color
        // -----------------------
        // darken by 10% like SASS https://github.com/gka/chroma.js/issues/217
        var brand_color_hover = chroma(brand_color).set("hsl.l", chroma(brand_color).get("hsl.l") - 0.1).hex();

        // 2) On Black and On White
        // -----------------------
        // compute the WCAG contrast ratio between two colors.
        // a minimum contrast of 4.5:1 is recommended to ensure that text is still readable against a background color.
        var brand_color_contrast_white = chroma.contrast('white', brand_color);
        var brand_color_contrast_black = chroma.contrast('black', brand_color);

        var brand_color_on_white = null;
        var brand_color_on_black = null;

        // console.log("contrast white: ", brand_color_contrast_white)
        // console.log("contrast black: ", brand_color_contrast_black)

        // check contrast on white
        if (brand_color_contrast_white < 4.5) {
            let hslPercent = 0;

            if (brand_color_contrast_white < 2) {
                hslPercent = 0.4;
            } else if (brand_color_contrast_white < 3) {
                hslPercent = 0.3;
            } else if (brand_color_contrast_white < 4) {
                hslPercent = 0.2;
            } else if (brand_color_contrast_white < 4.5) {
                hslPercent = 0.1;
            }

            // darken color by subtracting hslPercent
            // brand_color_on_white = chroma(brand_color).set("hsl.l", chroma(brand_color).get("hsl.l") - hslPercent).hex()

            // reduce luminance
            brand_color_on_white = chroma(brand_color).luminance(0.19).hex();
        }
        else {
            brand_color_on_white = brand_color;
        }

        if (brand_color_contrast_black < 4.5) {
            let hslPercent = 0;

            if (brand_color_contrast_black < 2) {
                hslPercent = 0.4;
            } else if (brand_color_contrast_black < 3) {
                hslPercent = 0.3;
            } else if (brand_color_contrast_black < 4) {
                hslPercent = 0.2;
            } else if (brand_color_contrast_black < 4.5) {
                hslPercent = 0.1;
            }

            // lighten color by adding hslPercent
            brand_color_on_black = chroma(brand_color).set("hsl.l", chroma(brand_color).get("hsl.l") + hslPercent).hex();
        }
        else {
            brand_color_on_black = brand_color;
        }

        // console.log("contrast brand_color_on_white: ", brand_color_on_white)
        // console.log("contrast brand_color_on_black: ", brand_color_on_black)

        // 3) Foreground text color
        // -------------------------
        // check whether brand_color is dark or light and whether the text color on top of it needs to be white or black
        // console.log("contrast lab.l: ", chroma(brand_color).get("lab.l"))
        // console.log("contrast luminance: ", chroma(brand_color).luminance())

        // lightness_threshold = 65
        var luminance_threshold = 0.2;

        // example colors for testing threshold
        // #f27949 (light: 64, lum: 0.33)
        // #ff00ff (light: 60, lum: 0.28)

        // if chroma(brand_color).get("lab.l") > lightness_threshold
        if (chroma(brand_color).luminance() > luminance_threshold) {
            // color is bright/light
            // console.log("contrast: brand_color is light")
            let css_body_color = getComputedStyle(document.documentElement).getPropertyValue('--body-color');
            css_body_color = css_body_color.trim(); // ! needs trimming

            if (css_body_color != '') {
                document.documentElement.style.setProperty('--brand-foreground-text-color', css_body_color);
                document.documentElement.style.setProperty('--brand-foreground-text-color-RGB', chroma(css_body_color).rgb().toString());
            }

            // override the default hover color, because we changed the foreground text color to black
            // therefore we want the hover color to be lighter instead of darker
            brand_color_hover = chroma(brand_color).set("hsl.l", chroma(brand_color).get("hsl.l") + 0.1).hex();
        }


        // 4) Set CSS Variables
        // -----------------------
        // need to apply style to the :root, because the CSS vars are defined in :root, :root will trump html
        // document.documentElement.style.setProperty('--brand-color', brand_color);
        document.documentElement.style.setProperty('--brand-color-RGB', chroma(brand_color).rgb().toString());
        document.documentElement.style.setProperty('--brand-color-hover', brand_color_hover);
        document.documentElement.style.setProperty('--brand-color-on-white', brand_color_on_white);
        document.documentElement.style.setProperty('--brand-color-on-black', brand_color_on_black);
    }
});